try:
    import cPickle as pickle
except:
    import pickle

import numpy as np
import seawater.gibbs as gsw

class Dict2Struc(object):
    """
    Open variables from a dictionary in a "matlab-like-structure"
    """
    def __init__(self, adict):
        self.__dict__.update(adict)

def test_print(method, comp_value=None):
    """
    Run a function test mimicking the original logic. This is done to allow for a direct comparison of the result from the Matlab to the python package.
    """
    width = 23
    if comp_value is None:
        comp_value = method

    # test if check value is identical to computed value
    if eval( "( gsw_cv." +comp_value+ "[~np.isnan(gsw_cv."+comp_value+")] == " +method+ "[~np.isnan("+method+")] ).all()" ):
        print "%s: Passed" % method.rjust(width)
    else:
        # test for floating differences with: computed - check value >= defined precision
        try:
            exec( "unequal = (gsw_cv." +comp_value+ " - " +method+ " ) >= gsw_cv." +comp_value+ "_ca" )
        except:
            exec( "unequal = (gsw_cv." +comp_value+ " - " +method+ " ) >= gsw_cv." +method+ "_ca" )
        if unequal.any():
            print "%s: Failed" % method.rjust(width)
        else:
            # for term25 and small float differences that will appear
            exec("nmax = ( (gsw_cv."+comp_value+" - "+method+")[~np.isnan(gsw_cv."+comp_value+")]).max()")
            exec("nmin = ( (gsw_cv."+comp_value+" - "+method+")[~np.isnan(gsw_cv."+comp_value+")]).min()")
            print "%s: Passed, but small diff ranging from: %s to %s" % ( method.rjust(width), nmax, nmin)

# load test data
data = pickle.load( open('data/gsw_cv.pkl','rb') )
gsw_cv = Dict2Struc(data) # then type dat.<tab> to navigate through your variables
# somehow longitude is store as uint8
gsw_cv.long_chck_cast = np.float64(gsw_cv.long_chck_cast)

# Test Section
""" Absolute Salinity (SA) and Preformed Salinity (Sstar) """
SA_chck_cast = gsw.SA_from_SP(gsw_cv.SP_chck_cast, gsw_cv.p_chck_cast, gsw_cv.long_chck_cast, gsw_cv.lat_chck_cast)[0]
test_print("SA_chck_cast", "SA_from_SP")

Sstar_from_SP = gsw.Sstar_from_SP(gsw_cv.SP_chck_cast, gsw_cv.p_chck_cast, gsw_cv.long_chck_cast, gsw_cv.lat_chck_cast)[0]
test_print("Sstar_from_SP")

SA_SA_Sstar_from_SP, Sstar_SA_Sstar_from_SP = gsw.SA_Sstar_from_SP(gsw_cv.SP_chck_cast, gsw_cv.p_chck_cast, gsw_cv.long_chck_cast, gsw_cv.lat_chck_cast)[0:2]
test_print("SA_SA_Sstar_from_SP")
test_print("Sstar_SA_Sstar_from_SP")

""" Conservative Temperature (CT) """
CT_chck_cast = gsw.conservative_t(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("CT_chck_cast", "CT_from_t")

""" other conversions between temperatures, salinities, pressure and height """
t_from_CT =  gsw.t_from_CT(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
test_print("t_from_CT", "t_chck_cast") #NOTE: diffs are also found in the original

pt = gsw.potential_t(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast,gsw_cv.pr)
test_print("pt", "pt_from_t")

CT_from_pt = gsw.CT_from_pt(SA_chck_cast, pt)
test_print("CT_from_pt") #NOTE: diffs are also found in the original

#pot_enthalpy = gsw.pot_enthalpy_from_pt(SA_chck_cast, pt)
#test_print("pot_enthalpy")

pt0 = gsw.pt0_from_t(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("pt0")

pt_from_CT = gsw.pt_from_CT(SA_chck_cast,CT_chck_cast)
test_print("pt_from_CT", "pt")

""" More on salinity """
Sstar_from_SA = gsw.Sstar_from_SA(SA_chck_cast, gsw_cv.p_chck_cast, gsw_cv.long_chck_cast, gsw_cv.lat_chck_cast)[0]
test_print("Sstar_from_SA")

SA_from_Sstar = gsw.SA_from_Sstar(SA_chck_cast, gsw_cv.p_chck_cast, gsw_cv.long_chck_cast, gsw_cv.lat_chck_cast)[0]
test_print("SA_from_Sstar")

SP_from_SA = gsw.SP_from_SA(SA_chck_cast, gsw_cv.p_chck_cast, gsw_cv.long_chck_cast, gsw_cv.lat_chck_cast)[0]
test_print("SP_from_SA", "SP_chck_cast") #NOTE: diffs are also found in the original

SP_from_Sstar = gsw.SP_from_SA(SA_chck_cast, gsw_cv.p_chck_cast, gsw_cv.long_chck_cast, gsw_cv.lat_chck_cast)[0]
test_print("SP_from_Sstar")

""" pressure /depth """
z_from_p = gsw.z_from_p(gsw_cv.p_chck_cast, gsw_cv.lat_chck_cast)
test_print("z_from_p")

p_from_z = gsw.p_from_z(z_from_p, gsw_cv.lat_chck_cast)
test_print("p_from_z") #FIXME: diffs are not found in the original

""" more temperatures conversions """
from seawater.csiro import T90conv
t90_from_t68 = T90conv(gsw_cv.t_chck_cast, t_type='T68')
test_print("t90_from_t68")

t90_from_t48 = T90conv(gsw_cv.t_chck_cast, t_type='T48')
test_print("t90_from_t48")

""" density and enthalpy,  based on the 25-term expression for density """
#rho_CT25 = gsw.rho_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("rho_CT25")

#rho_CT25rab, alpha_CT25rab, beta_CT25rab = gsw.rho_alpha_beta_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("rho_CT25rab")
#test_print("alpha_CT25rab")
#test_print("beta_CT25rab")

#specvol_CT25 = gsw.specvol_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("specvol_CT25")

#specvol_anom_CT25 = gsw.specvol_anom_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("specvol_anom_CT25")

#enthalpy_CT25 =  gsw.enthalpy_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("enthalpy_CT25")

#enthalpy_diff_CT25 =  gsw.enthalpy_diff_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast_shallow, gsw_cv.p_chck_cast_deep)
#test_print("enthalpy_diff_CT25")

""" water column properties,  based on the 25-term expression for density """
#n2, p_mid_n2 = gsw.Nsquared_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast, gsw_cv.lat_chck_cast)
#test_print("n2")
#test_print("p_mid_n2")

#Tu, Rsubrho, p_mid_TuRsr = gsw.Turner_Rsubrho_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("Tu")
#test_print("Rsubrho")
#test_print("p_mid_TuRsr")

#IPVfN2, p_mid_IPVfN2 = gsw.IPV_vs_fNsquared_ratio_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast, gsw_cv.pr)
#test_print("IPVfN2")
#test_print("p_mid_IPVfN2")

""" geostrophic streamfunctions,  based on the 25-term expression for density """
#geo_strf_dyn_height = gsw.geo_strf_dyn_height(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("geo_strf_dyn_height")

#geo_strf_dyn_height_pc, dh_pmid = gsw.geo_strf_dyn_height_pc(SA_chck_cast, CT_chck_cast, gsw_cv.delta_p_chck_cast)
#test_print("geo_strf_dyn_height_pc")
#test_print("dh_pmid")

#geo_strf_McD_Klocker = gsw.geo_strf_McD_Klocker(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast, gsw_cv.Neutral_Density, gsw_cv.p_Neutral_Density)
#test_print("geo_strf_McD_Klocker")

#geo_strf_McD_Klocker_pc, mk_p_mid = gsw.geo_strf_McD_Klocker_pc(SA_chck_cast, CT_chck_cast, gsw_cv.delta_p_chck_cast, gsw_cv.Neutral_Density[0], 3)
#test_print("geo_strf_McD_Klocker_pc")
#test_print("mk_p_mid")

#geo_strf_Montgomery = gsw.geo_strf_Montgomery(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("geo_strf_Montgomery")

#geo_strf_Cunningham = gsw.geo_strf_Cunningham(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("geo_strf_Cunningham")

#geo_str_velocity, gv_mid_lat, gv_mid_long = gsw.geostrophic_velocity(geo_strf_dyn_height, gsw_cv.long_chck_cast, gsw_cv.lat_chck_cast, gsw_cv.p_chck_cast)
#test_print("geo_str_velocity")
#test_print("gv_mid_lat")
#test_print("gv_mid_long")

""" neutral and non-linear properties,  based on the 25-term expression for density """
#cabbeling_CT25 = gsw.cabbeling_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("cabbeling_CT25")

#thermobaric_CT25 = gsw.thermobaric_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("thermobaric_CT25")

#isopycnal_slope_ratio_CT25 = gsw.isopycnal_slope_ratio_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast, gsw_cv.pr)
#test_print("isopycnal_slope_ratio_CT25")

#G_CT_CT25, p_mid_G_CT_CT25 = gsw.isopycnal_vs_ntp_CT_ratio_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast, gsw_cv.pr)
#test_print("G_CT_CT25")
#test_print("p_mid_G_CT_CT25")

#ntpptCT_CT25 = gsw.ntp_pt_vs_CT_ratio_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("ntpptCT_CT25")

""" basic thermodynamic properties """
rho = gsw.rho(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("rho")

pot_rho = gsw.pot_rho(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast, gsw_cv.pr)
test_print("pot_rho")

specvol = gsw.specvol(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("specvol")

specvol_anom = gsw.specvol_anom(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("specvol_anom")

alpha_wrt_CT = gsw.alpha_wrt_CT(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("alpha_wrt_CT")

alpha_wrt_pt = gsw.alpha_wrt_pt(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("alpha_wrt_pt")

alpha_wrt_t = gsw.alpha_wrt_t(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("alpha_wrt_t")

beta_const_CT = gsw.beta_const_CT(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("beta_const_CT")

beta_const_pt = gsw.beta_const_pt(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("beta_const_pt")

beta_const_t = gsw.beta_const_t(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("beta_const_t")

entropy = gsw.entropy(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("entropy")

internal_energy = gsw.internal_energy(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("internal_energy")

enthalpy = gsw.enthalpy(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("enthalpy")

cp = gsw.cp(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("cp")

isochoric_heat_cap = gsw.isochoric_heat_cap(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("isochoric_heat_cap")

chem_potential =  gsw.chem_potential_relative(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("chem_potential")

chem_potential_water =  gsw.chem_potential_water(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("chem_potential_water")

chem_potential_salt =  gsw.chem_potential_salt(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("chem_potential_salt")

Helmholtz_energy = gsw.helmholtz_energy(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("Helmholtz_energy")

sound_speed = gsw.sound_speed(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("sound_speed")

kappa = gsw.kappa(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("kappa")

kappa_const_t = gsw.kappa_const_t(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("kappa_const_t")

adiabatic_lapse_rate = gsw.adiabatic_lapse_rate(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("adiabatic_lapse_rate")

molality = gsw.molality(SA_chck_cast)
test_print("molality")

ionic_strength = gsw.ionic_strength(SA_chck_cast)
test_print("ionic_strength")

osmotic_coefficient = gsw.osmotic_coefficient(SA_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("osmotic_coefficient")

#t_maxden, pt_maxden, CT_maxden = gsw.temps_maxdensity(SA_chck_cast, gsw_cv.p_chck_cast)
#test_print("t_maxden")
#test_print("pt_maxden")
#test_print("CT_maxden")

""" basic thermodynamic properties in terms of CT and pt """
#rho_CT = gsw.rho_CT(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("rho_CT")

#rho_CTrab, alpha_CTrab, beta_CTrab = gsw.rho_alpha_beta_CT(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("rho_CTrab")
#test_print("alpha_CTrab")
#test_print("beta_CTrab")

#specvol_CT = gsw.specvol_CT25(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("specvol_CT")

#specvol_anom_CT = gsw.specvol_anom_CT(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("specvol_anom_CT")

#sigma0_CT = gsw.sigma0_CT(SA_chck_cast, CT_chck_cast)
#test_print("sigma0_CT")

#sigma1_CT = gsw.sigma1_CT(SA_chck_cast, CT_chck_cast)
#test_print("sigma1_CT")

#sigma2_CT = gsw.sigma2_CT(SA_chck_cast, CT_chck_cast)
#test_print("sigma2_CT")

#sigma3_CT = gsw.sigma3_CT(SA_chck_cast, CT_chck_cast)
#test_print("sigma3_CT")

#sigma4_CT = gsw.sigma4_CT(SA_chck_cast, CT_chck_cast)
#test_print("sigma4_CT")

#enthalpy_CT =  gsw.enthalpy_CT(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("enthalpy_CT")

#enthalpy_diff_CT =  gsw.enthalpy_diff_CT(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast_shallow, gsw_cv.p_chck_cast_deep)
#test_print("enthalpy_diff_CT")

entropy_from_pt =  gsw.entropy_from_t(SA_chck_cast, pt, t_type='pt')
test_print("entropy_from_pt") #NOTE: diffs are also found in the original

entropy_from_CT =  gsw.entropy_from_t(SA_chck_cast, CT_chck_cast, t_type='CT')
test_print("entropy_from_CT")

CT_from_entropy =  gsw.t_from_entropy(SA_chck_cast, entropy, t_type='CT')
test_print("CT_from_entropy") #FIXME: diffs are not found in the original

pt_from_entropy =  gsw.t_from_entropy(SA_chck_cast, entropy, t_type='pt')
test_print("pt_from_entropy") #FIXME: diffs are not found in the original

""" derivatives of enthalpy,  entropy,  CT and pt """
#[CT_SA,  CT_pt] = gsw.CT_first_derivatives(SA_chck_cast, pt)
#test_print("CT_SA")
#test_print("CT_pt")

#CT_SA_SA, CT_SA_pt, CT_pt_pt = gsw.CT_second_derivatives(SA_chck_cast, pt)
#test_print("CT_SA_SA")
#test_print("CT_SA_pt")
#test_print("CT_pt_pt")

#h_SA, h_CT, h_P = gsw.enthalpy_first_derivatives(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("h_SA")
#test_print("h_CT")
#test_print("h_P")

#h_SA_SA, h_SA_CT, h_CT_CT = gsw.enthalpy_second_derivatives(SA_chck_cast, CT_chck_cast, gsw_cv.p_chck_cast)
#test_print("h_SA_SA")
#test_print("h_SA_CT")
#test_print("h_CT_CT")

#eta_SA, eta_CT = gsw.entropy_first_derivatives(SA_chck_cast, CT_chck_cast)
#test_print("eta_SA")
#test_print("eta_CT")

#eta_SA_SA, eta_SA_CT, eta_CT_CT = gsw.entropy_second_derivatives(SA_chck_cast, CT_chck_cast)
#test_print("eta_SA_SA")
#test_print("eta_SA_CT")
#test_print("eta_CT_CT")

#pt_SA,  pt_CT = gsw.pt_first_derivatives(SA_chck_cast, CT_chck_cast)
#test_print("pt_SA")
#test_print("pt_CT")

#pt_SA_SA, pt_SA_CT, pt_CT_CT = gsw.pt_second_derivatives(SA_chck_cast, CT_chck_cast)
#test_print("pt_SA_SA")
#test_print("pt_SA_CT")
#test_print("pt_CT_CT")

""" planet earth properties """
from seawater.csiro import cor
f = cor(gsw_cv.lat_chck_cast)
test_print("f")

grav = gsw.grav(gsw_cv.lat_chck_cast, gsw_cv.p_chck_cast)
test_print("grav")

distance = gsw.distance(gsw_cv.long_chck_cast, gsw_cv.lat_chck_cast, gsw_cv.p_chck_cast)
test_print("distance") #FIXME: diffs are not found in the original

""" Absolute Salinity from direct density measurements:- a laboratory function """
SA_from_rho = gsw.SA_from_rho(rho, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("SA_from_rho")

#sigma0_pt = gsw.sigma0_pt(SA_chck_cast, pt0)
#test_print("sigma0_pt")

""" Practical Salinity (SP):- PSS-78 """
cndr = gsw.cndr_from_SP(gsw_cv.SP_chck_cast, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("cndr") #FIXME: diffs are not found in the original

SP_from_cndr = gsw.SP_from_cndr(cndr, gsw_cv.t_chck_cast, gsw_cv.p_chck_cast)
test_print("SP_from_cndr") #FIXME: diffs are not found in the original


"""
Constants used
==============
"""

C3515 = 42.914 #NOTE: reference value, not used.
"""
Conductivity of 42.914 [mmho cm :sup:`-1` == mS cm :sup:`-1`] at Salinity 35 psu, Temperature 15 :math:`^\\circ` C [ITPS 68] and Pressure 0 db.

References
----------
.. [1] R.C. Millard and K. Yang 1992. "CTD Calibration and Processing Methods used by Woods Hole Oceanographic Institution" Draft April 14, 1992 (Personal communication)
"""

a = 6371000.
"""
mean radius of earth  A.E. Gill
"""

OMEGA = 7.292115e-5
"""
:math:`\\Omega = \\frac{2\\pi}{\\textrm{sidereal day}}` = 7.292e-5.radians sec :sup:`-1`

1 sidereal day = 23.9344696 hours

Changed to a more precise value at Groten 2004

References
----------
.. [1] A.E. Gill 1982. p.54  eqn 3.7.15 "Atmosphere-Ocean Dynamics" Academic Press: New York. ISBN: 0-12-283522-0. page: 597
.. [2] Groten, E., 2004: Fundamental Parameters and Current (2004) Best Estimates of the Parameters of Common Relevance to Astronomy, Geodesy, and Geodynamics. Journal of Geodesy, 77, pp. 724-797.
"""

gdef = 9.8
"""
Acceleration of gravity [m s :sup:`2`] used by sw.swvel and bfrq without lat info.
"""

DEG2NM = 60
NM2KM = 1.8520
"""
1 nm = 1.8520 km
Used by sw.dist() to convert nautical miles to kilometers.

References
----------
.. [1] S. Pond & G.Pickard 2nd Edition 1986 Introductory Dynamical Oceanogrpahy Pergamon Press Sydney. ISBN 0-08-028728-X. page: 303
"""

Kelvin = 273.15
"""
offset to convert :math:`^\\circ` C to Kelvin. Used in the gas solubility functions.
"""

db2Pascal = 1e4
"""
Decibar to pascal
"""

#--- only for gsw
gamma = 2.26e-7 #TODO: define
"""
Gamma (A.E. Gill)
"""

M_S = 0.0314038218
"""
mole-weighted average atomic weight of the elements of sea salt, in units of kg mol :sup:`-1`
"""

cp0 = 3991.86795711963 #TODO: define
"""
from Eqn. (3.3.3) of IOC et al. (2010).
"""

SSO = 35.16504 #TODO: define
"""
from section 2.4 of IOC et al. (2010)
"""

sfac = 0.0248826675584615 #TODO: define
"""
sfac = 1 / (40 * ( SSO / 35 ) )
"""

R = 8.314472
"""
the molar gas constant = 8.314472 m :sup:`2` kg s:sup:`-21 K :sup:`-1` mol :sup:`-1`
"""

r1 = 0.35
"""
TODO
"""


# -*- coding: utf-8 -*-

"""
Python Seawater
===============

This python package contains:
1) Python translation of the original CSIRO Matlab package (SEAWATER-3.3) for calculating the properties of sea water. The package uses the formulas from Unesco's joint panel on oceanographic tables and standards, UNESCO 1981 and UNESCO 1983.

2) Python translation of the original Gibbs-SeaWater Matlab package (GSW v2.0). The Gibbs-SeaWater (GSW) Oceanographic Toolbox contains the TEOS-10 subroutines for evaluating the thermodynamic properties of pure water (using IAPWS-09) and seawater (using IAPWS-08 for the saline part).

The author has no intention to do things in a "pythonic-way", it is just a "work around" from someone that couldn't afford Matlab anymore.

CAVEAT: These modules do not adhere to strict basic-SI units but rather oceanographic units are adopted.
"""

"""
Sea pressure is absolute pressure - 10.1325 dbar (or minus atmospheric pressure)
"""

"""
For the first time the influence of the spatially varying composition of seawater can be systematically taken into account through the use of Absolute Salinity. In the open ocean, this has a non-trivial effect on the horizontal density gradient, and thereby on the ocean velocities and heat transports calculated via the thermal wind relationship.
"""

"""
The new salinity variable, Absolute Salinity, is measured in SI units (e.g. g/kg).
"""

"""
The thermodynamic quantities available from TEOS-10 are totally consistent with each other, while this was not the case with EOS-80
"""

"""
The new equation incorporates a more accurate representation of salinity known as Absolute Salinity. The main justification for preferring Absolute Salinity over the most recent salinity definition, Practical Salinity, is that the thermodynamic properties of seawater are directly influenced by the total mass of dissolved constituents (Absolute Salinity). However, the mass of dissolved constituents are regionally variable and are not always accurately represented when using conductivity measurements of seawater, the key parameter used in the calculation of Practical Salinity.
"""

"""
In addition, it would be useful to have a conservative tracer for salinity. The Absolute Salinity SA is not conservative, because it slowly increases in the ocean due to biogeochemical processes, which remineralize carbon and nutrients. The Reference Salinity SR is also not conservative, since it is measured with conductivity which will also slowly increase due to these increases in concentrations of nutrient and carbon system ions.

However, we can "remove" these effects and construct a conservative salinity tracer Sstar..

Typically this PREFORMED SALINITY Sstar will equal SR (and SA) in Standard Seawater (SSW), but as ions are added it will become smaller than either SR and SA (as illustrated by the example in Table 1).
"""

"""
CT == \BigTheta = frac{h_o}{C^o_p}.

The scale factor C^o_p is a constant carefully chosen so that potential temperature \smalltheta and Conservative Temperature \BigTheta will be numerically similar for typical seawater at SP = 35, or near t = 0 degC. However, the difference between the two can exceed 1◦C when salinities are low and temperatures high (for details, see IOC et al., 2010).
"""

__authors__    = ['Filipe Fernandes']
__license__    = ["MIT"]
__version__    = ["2.0.0"]
__maintainer__ = ["Filipe Fernandes"]
__email__      = ["ocefpaf@gmail.com"]
__status__     = ["Production"]
__created__    = ["14-Jan-2010"]
__modified__   = ["19-Feb-2011"]
__all__        = ["seawater","gibbs","csiro","extras"] #FIXME


# -*- coding: utf-8 -*-

import numpy as np
import seawater.csiro as sw

class Waves():
    """
    Solves the wave dispersion relationship.

    .. math::
        \Omega = ...

    Parameters
    ----------
    h : array_like, str
        Water depth [m] or 'deep', 'shallow' as keywords
    T : array_like
        Wave period [s]
    L : array_like
        Wave length [m]
    thetao : array_like
        TODO
    Ho : array_like
        TODO

    Returns
    -------
    omega : array_like
            Wave frequency
    TODO: hoLo, hoL, Lo, L, k, T, Co, C, Cg, G, Ks, Kr, theta, H

    Notes
    -----
    Compare values with:
    http://www.coastal.udel.edu/faculty/rad/wavetheory.html

    Examples
    --------
    >>> from seawater.extras.waves import Waves
    >>> Waves(h=10, T=5, L=None)

    References
    ----------
    TODO

    Modifications: Filipe Fernandes, 2010
                10-01-26. Filipe Fernandes, first version.
    """

    def __init__(self, h, T=None, L=None, thetao=None, Ho=None, lat=None):
        self.T = np.asarray( T, dtype=np.float32 )
        self.L = np.asarray( L, dtype=np.float32 )
        self.Ho = np.asarray( Ho, dtype=np.float32 )
        self.lat = np.asarray( lat, dtype=np.float32 )
        self.thetao = np.asarray( thetao, dtype=np.float32 )

        if isinstance(h, str):
            if L is not None:
                if h == 'deep':
                    self.h = self.L/2.
                elif h == 'shallow':
                    self.h = self.L * 0.05
        else:
            self.h = np.asarray( h, dtype=np.float32 )

        if lat is None:
            g = 9.81 # default gravity
        else:
            g = sw.grav(lat) #FIXME

        if L is None:
            self.omega = 2 * np.pi / self.T
            self.Lo    = (g * self.T**2) / 2 / np.pi
            # returns wavenumber of the gravity wave dispersion relation using newtons method
            self.k = self.omega / np.sqrt(g) # the initial guess will be the shallow water wavenumber
            # TODO: May change to : self.k = self.w**2 / ( g * np.sqrt(
            # self.w**2 * self.h / g ) )
            f = g * self.k * np.tanh( self.k * self.h ) - self.omega**2

            while np.abs(f.max()) > 1e-10:
                dfdk = g * self.k * self.h * ( 1 / ( np.cosh( self.k * self.h ) ) )**2 + g * np.tanh( self.k * self.h )
                self.k    = self.k - f / dfdk
                f    = g * self.k * np.tanh( self.k * self.h ) - self.omega**2 #FIXME

            self.L = 2 * np.pi / self.k
            if isinstance(h, str):
                if h == 'deep':
                    self.h = self.L/2.
                elif h == 'shallow':
                    self.h = self.L * 0.05
        else:
            self.Lo    = self.L / np.tanh( 2 * np.pi * self.h / self.L )
            self.k     = 2 * np.pi / self.L
            self.T     = np.sqrt( 2 * np.pi * self.Lo / g )
            self.omega = 2 * np.pi / self.T

        self.hoL   = self.h / self.L
        self.hoLo  = self.h / self.Lo
        self.C     = self.omega / self.k # or L / T
        self.Co    = self.Lo / self.T
        self.G     = 2 * self.k * self.h / np.sinh( 2 * self.k * self.h )
        self.n     = ( 1 + self.G ) / 2
        self.Cg    = self.n * self.C
        self.Ks    = np.sqrt( 1 / ( 1 + self.G ) / np.tanh( self.k * self.h ) )

        if thetao is None:
            self.theta = np.NaN
            self.Kr    = np.NaN
        if thetao is not None:
            self.theta = np.rad2deg( np.asin( self.C / self.Co * np.sin( np.deg2rad( self.thetao ) ) ) )
            self.Kr    = np.sqrt( np.cos( np.deg2rad( self.thetao ) ) / np.cos( np.deg2rad( self.theta ) ) )

        if Ho is None:
            self.H = np.NaN
        if Ho is not None:
            self.H = self.Ho * self.Ks * self.Kr

if __name__=='__main__':
    wav = Waves(h=10, T=5, L=None)
    print "ho/Lo = %s " % wav.hoLo
    print "ho/L  = %s " % wav.hoL
    print "Lo    = %s " % wav.Lo
    print "L     = %s " % wav.L
    print "k     = %s " % wav.k
    print "omega = %s " % wav.omega
    print "T     = %s " % wav.T
    print "C     = %s " % wav.C
    print "Cg    = %s " % wav.Cg
    print "G     = %s " % wav.G
    #print "  = %s " % wav.G
    #print "  = %s " % wav.Ks
    #print "  = %s " % wav.Kr
    #print "  = %s " % wav.theta
    #print "  = %s " % wav.H
    print
    print
    wav = Waves(h=10, T=None, L=100)
    print "ho/Lo = %s " % wav.hoLo
    print "ho/L  = %s " % wav.hoL
    print "Lo    = %s " % wav.Lo
    print "L     = %s " % wav.L
    print "k     = %s " % wav.k
    print "omega = %s " % wav.omega
    print "T     = %s " % wav.T
    print "C     = %s " % wav.C
    print "Cg    = %s " % wav.Cg
    print "G     = %s " % wav.G
    #print "  = %s " % wav.G
    #print "  = %s " % wav.Ks
    #print "  = %s " % wav.Kr
    #print "  = %s " % wav.theta
    #print "  = %s " % wav.H


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

classifiers = """\
Development Status :: 5 - Production/Stable
Environment :: Console
Intended Audience :: Science/Research
Intended Audience :: Developers
Intended Audience :: Education
License :: OSI Approved :: MIT License
Operating System :: OS Independent
Programming Language :: Python
Topic :: Scientific/Engineering
Topic :: Education
Topic :: Software Development :: Libraries :: Python Modules
"""

setup(name             = 'seawater',
      version          = '2.0.0',
      author           = 'Filipe Fernandes',
      author_email     = 'ocefpaf@gmail.com',
      maintainer       = 'Filipe Fernandes',
      maintainer_email = 'ocefpaf@gmail.com',
      url              = 'http://pypi.python.org/pypi/seawater/',
      description      = 'Seawater Libray for Python',
      long_description = """\
This python package contains a python translation for two Matlab user contributed toolboxes. (1) the `seawater <http://www.cmar.csiro.au/datacentre/ext_docs/seawater.htm>`_ (EOS-80) and (2) the `gibbs <http://www.teos-10.org/software.htm>`_ seawater (TEOS-10).
""",
      download_url     = 'http://pypi.python.org/packages/source/s/seawater/',
      classifiers      = filter(None, classifiers.split("\n")),
      platforms        = 'any',
      packages = find_packages(),
      package_data     = {'':['data/*.pkl']},
      zip_safe = False,
      license          = 'MIT',
      keywords         = 'oceanography seawater',
    )
